import router from '@/router'
import store from '@/store'

import {Dialog} from 'vant'
import {gotoMobileLoginPage, gotoWechatOauth2Page} from '../utils/auth'
import {getValueByKey, removeTokenByKey, removeUserInfo, removeValueByKey, setValueByKey} from '../utils/cache/auth'
import {STRING_LOGIN_OAUTH2_CODE_KEY, STRING_LOGIN_SESSION_ID_KEY, STRING_LOGIN_TICKET_KEY, STRING_USER_INFO_KEY, STRING_USER_TOKE_KEY} from '../constants/auth'
import * as dd from 'dingtalk-jsapi'
import {getUrlParamByKey, sortUrlParam} from '../utils/util.url'
import {normalLogin, validateLocalLoginInfo} from 'string-public-design-3/utils/util.permission'
import {showVersion} from "../utils/util.version";

/**
 * 达科登录
 * 登录流程：进入应用，地址栏会携带token（并非业务系统的token，相当于微信认证中心重定向返回的code），拿token登录换取用户信息
 * 存在问题：用户登录过期后无法重新登录，需要退出应用重新进入
 * */
async function loginByDatatech(authConfig, token = null) {
    return new Promise(async resolve => {
        if (!token) token = getUrlParamByKey('token')
        if (token) {
            console.log('获取到地址栏token >>> ' + token)
            //用token登录换取用户信息
            await store.dispatch('user/mobileLogin', {token}).catch(ex => {
                Dialog.confirm({
                    title: '提示',
                    message: '登录失败，请稍后重试'
                }).then(() => {
                    loginByDatatech(authConfig, token)
                }).catch(() => {
                })
            })
        } else {
            console.log('未获取到地址栏token')
            if (!validateLocalLoginInfo()) {
                Dialog.alert({
                    title: '提示',
                    message: '未获取到用户登录信息，请退出应用重试'
                })
            } else {
                // 验证当前token是否过期
                await store.dispatch('user/checkTicketValid').catch(ex => {
                    console.log(ex)
                })
                // 查看当前验证的结果
                if (store.state.user.isTokenValid == false) {
                    Dialog.alert({
                        title: '提示',
                        message: '登录已过期，请退出应用重试'
                    })
                }
            }
        }
        resolve()
    })
}

/**
 * 钉钉登录
 * 登录流程：进入应用，判断本地是否存在token，存在则验证token有效性，不存在则清除缓存并获取code重新登录
 * */
async function loginByDing(authConfig) {
    //清除缓存并获取钉钉登录code
    function removeCacheAndGetDingLoginCode() {
        removeUserInfo(STRING_USER_INFO_KEY)
        return new Promise((resolve, reject) => {
            dd.ready(() => {
                dd.runtime.permission.requestAuthCode({
                    corpId: authConfig.mobileDingCorpId,// window.string.STRING_DING_CORP_ID
                    onSuccess: function (result) {
                        let code = result.code
                        console.log('获取登录code成功 >>>' + code)
                        resolve(code)
                    },
                    onFail: function () {
                        console.log('获取登录code失败')
                        reject()
                    }
                })
            })
        })
    }

    //登录
    async function login(code) {
        return new Promise(async resolve => {
            await store.dispatch('user/loginDing', {dingCode: code}).catch(ex => {
                Dialog.confirm({
                    title: '提示',
                    message: '登录失败，请稍后重试'
                }).then(() => {
                    loginByDing(authConfig)
                }).catch(() => {
                })
            })
            resolve()
        })
    }

    return new Promise(async resolve => {
        if (!validateLocalLoginInfo()) {
            const code = await removeCacheAndGetDingLoginCode()
            await login(code)
        } else {
            // 验证当前token是否过期
            await store.dispatch('user/checkTicketValid').catch(ex => {
                console.log(ex)
            })
            // 查看当前验证的结果
            if (store.state.user.isTokenValid == false) {
                const code = await removeCacheAndGetDingLoginCode()
                await login(code)
            }
        }
        resolve()
    })
}

/**
 * 企业微信登录
 * 登录流程：进入应用，判断地址栏是否存在code
 *          如果存在，直接用code登录换取用户信息
 *          不存在，判断本地是否存在token，存在则验证token有效性，不存在则清除缓存并跳转到微信认证中心
 * */
async function loginByWechat(authConfig) {
    //清除缓存并跳转到微信认证中心
    function removeCacheAndGotoWechatOauth2Page() {
        removeUserInfo(STRING_USER_INFO_KEY)
        gotoWechatOauth2Page(authConfig.mobileWechatAppId)
    }

    return new Promise(async resolve => {
        const isIgnore = getUrlParamByKey('isIgnore')
        if (isIgnore) return resolve()//忽略当前登录
        const code = getUrlParamByKey('code')
        if (code) {
            console.log('获取到企业微信登录code >>> ' + code)
            setValueByKey(STRING_LOGIN_OAUTH2_CODE_KEY, code)
            console.log('获取到企业微信登录code >>> ' + code)
            //用code登录换取用户信息
            await store.dispatch('user/mobileLogin', {wechatCode: code}).catch(ex => {
                Dialog.confirm({
                    title: '提示',
                    message: '登录失败，请稍后重试'
                }).then(() => {
                    removeCacheAndGotoWechatOauth2Page()
                }).catch(() => {
                })
            })
        } else {
            console.log('未获取到企业微信登录code')
            if (!validateLocalLoginInfo()) {
                removeCacheAndGotoWechatOauth2Page()
            } else {
                // 验证当前token是否过期
                await store.dispatch('user/checkTicketValid').catch(ex => {
                    console.log(ex)
                })
                // 查看当前验证的结果
                if (store.state.user.isTokenValid == false) {
                    removeCacheAndGotoWechatOauth2Page()
                }
            }
        }
        resolve()
    })
}

/**
 * H5登录
 * 登录流程：判断地址栏是否存在ticket，如果存在，则是已经登录完成重定向回来，直接使用ticket登录并换取用户信息
 *          如果不存在，则判断缓存中是否存在登录信息，如果有，验证有效性，没有则跳转到登录页
 * */
async function loginByH5() {
    return new Promise(async resolve => {
        const ticket = getUrlParamByKey()
        if (ticket) {
            //获取本地的ticket
            let localTicket = getValueByKey(STRING_LOGIN_TICKET_KEY)
            if (localTicket && localTicket === ticket) {
                // 校验当前登录有效性
                await store.dispatch('user/checkTicketValid').catch(ex => {
                    console.log(ex)
                })
            } else {
                // 使用ticket获取用户信息
                let resToken = await store.dispatch('user/getUserInfoByTicket', ticket).catch(ex => {
                    Dialog.confirm({
                        title: '提示',
                        message: '获取用户信息失败，请重新登录',
                        confirmButtonText: '立即登录'
                    }).then(() => {
                        store.dispatch('user/logoutMobile').catch(ex => {
                            console.log(ex)
                        })
                    }).catch(() => {
                    })
                    resToken = 'relogin'
                })
                if (resToken === 'relogin') return
                if (!resToken) {
                    Dialog.confirm({
                        title: '提示',
                        message: '当前用户登录状态异常或已被禁用，请更换用户重新登录',
                        confirmButtonText: '重新登录'
                    }).then(() => {
                        // 清除缓存
                        removeTokenByKey(STRING_USER_TOKE_KEY)
                        removeUserInfo(STRING_USER_INFO_KEY)
                        removeValueByKey(STRING_LOGIN_SESSION_ID_KEY)
                        //前往退出登录页面
                        gotoMobileLoginPage()
                    }).catch(() => {
                    })
                }
                resolve()
            }
        } else {
            if (!validateLocalLoginInfo()) {
                await store.dispatch('user/logout').catch(ex => {
                    console.log(ex)
                })
            } else {
                // 校验当前登录有效性
                await store.dispatch('user/checkTicketValid').catch(ex => {
                    console.log(ex)
                })
                resolve()
            }
        }
    })
}

// 路由之前
router.beforeEach(async (to, from, next) => {

    // 检查是否为白名单页面
    if (window.string.ROUTE_WHITE_LIST.indexOf(to.path) !== -1) {
        return next()
    }

    // 检查页面是否需要验证
    if ((to.meta.isAuth && to.meta.isAuth == true)) {
        return next()
    }

    // 单点登录处理参数顺序
    sortUrlParam()

    // 获取基础配置
    let startConfigTimestamp = new Date().getTime()
    await store.dispatch('config/getData').catch(ex => {
        console.log(ex)
    })
    let endConfigTimestamp = new Date().getTime()
    console.log(`==========获取基础配置 ${endConfigTimestamp - startConfigTimestamp}===========`)
    //移动端登录逻辑
    let authConfig = store.getters['config/authConfig']
    //判断是否第三方登录
    switch (authConfig.mobileLoginMethod) {
        case 'DING'://钉钉
            await loginByDing(authConfig)
            break
        case 'WECHAT'://企业微信
            await loginByWechat(authConfig)
            break
        case 'DATATECH'://达科
            await loginByDatatech(authConfig)
            break
        case 'H5'://H5
            await loginByH5(authConfig)
            break
        default:
            Dialog.alert({title: '提示', message: '暂不支持钉钉、达科、企业微信之外的登录方式'})
            break
    }

    //打印版本信息
    showVersion();

    // 进入页面
    let query = to.query
    if ('token' in query) {//达科登录地址栏会有token，客户要求去掉
        delete query.token
        next({path: to.path, query})
    } else {
        next()
    }
})
