import {getAppByKey, saveApp, cancelSaveApp} from 'string-public-design-3/api/framework/app/index'
import {queryDesktopAppsByCode} from 'string-public-design-3/api/framework/app/desktop'
import {removePerms} from 'string-public-design-3/utils/cache/auth.js'

import {
    setPerms,
    getPerms as getPermsFromCache,
    setCurrentPerm,
    getCurrentPerm,
} from "string-public-design-3/utils/cache/auth"

import {
    STRING_PERMISSION_KEY,
    STRING_CURRENT_PERMISSION_KEY
} from "string-public-design-3/constants/auth"


import {getAppClientInfo} from 'string-public-design-3/methods/app'

import {getBaseData} from "string-public-design-3/methods/cache"
import {setData} from "string-public-design-3/utils/cache/db"

const appTypes = [
    {id: "1", name: "流程应用"},
    {id: "2", name: "数据模板"},
    {id: "4", name: "自研应用"},
    {id: "5", name: "内置应用"},
    {id: "6", name: "报表应用"},
    {id: "7", name: "第三方应用"},
]

const state = {
    db: null,

    // 应用类型列表
    appTypes,

    // 应用标签树形结构
    appTagTree: [],
    // 应用类型树形结构
    appCategoryTree: [],
    // 应用树形结构
    appTree: [],
    // 流程类应用树形结构
    workflowAppTree: [],

    // 授权的应用
    assignedApps: [],
    // 收藏的应用
    favoriteApps: [],
    // 推荐的应用
    recommendedApps: [],
    //最近使用的应用
    recentUseApps: [],

    // 当前的应用：查看应用详情时使用 TODO:即将废弃，当不再使用框架内部的头和网站地图之后，就不用了。
    currentApp: null,

    // 当前访问的应用
    visitApp: null,
    // 当前访问的应用权限列表
    visitAppPermissions: getPermsFromCache(STRING_PERMISSION_KEY),
    // 当前访问的应用当前权限
    visitAppCurrentPerm: getCurrentPerm(STRING_CURRENT_PERMISSION_KEY),
}

const getters = {
    // 基础数据
    apps: state => state.db ? state.db.find(item => item.code == "APP").value : [],
    appTags: state => state.db ? state.db.find(item => item.code == "APP_TAG").value : [],
    appCategories: state => state.db ? state.db.find(item => item.code == "APP_CATEGORY").value : [],

    // 应用分类
    appTypes: state => state.appTypes,
    // 树形结构
    appTagTree: state => state.appTagTree,
    appCategoryTree: state => state.appCategoryTree,
    appTree: state => state.appTree,
    // 流程类应用树形结构，为一网通办准备。
    workflowAppTree: state => state.workflowAppTree,

    // 授权的、收藏的、推荐的、最近使用的应用
    assignedApps: state => state.assignedApps,
    favoriteApps: state => state.favoriteApps,
    recommendedApps: state => state.recommendedApps,
    recentUseApps: state => state.recentUseApps,

    // 当前的应用, TODO:即将废弃，当不再使用框架内部的头和网站地图之后，就不用了。
    currentApp: state => state.currentApp,

    // 当前访问的应用
    visitApp: state => state.visitApp,
    // 当前访问的应用权限列表
    visitAppPermissions: state => state.visitAppPermissions,
    // 当前访问的应用当前权限
    visitAppCurrentPerm: state => state.visitAppCurrentPerm,

}

const mutations = {
    SET_DB: (state, db) => {
        state.db = db
    },

    SET_APP_CATEGORY_TREE(state, appCategoryTree) {
        state.appCategoryTree = appCategoryTree;
    },
    SET_APP_TAG_TREE(state, appTagTree) {
        state.appTagTree = appTagTree;
    },
    SET_APP_TREE(state, appTree) {
        state.appTree = appTree;
    },
    SET_WORKFLOW_APP_TREE(state, workflowAppTree) {
        state.workflowAppTree = workflowAppTree;
    },


    SET_ASSIGNED_APPS(state, apps) {
        state.assignedApps = apps
    },
    SET_FAVORITE_APPS(state, apps) {
        state.favoriteApps = apps
    },
    SET_RECOMMENDED_APPS(state, apps) {
        state.recommendedApps = apps;
    },
    SET_RECENT_USE_APPS(state, apps) {
        state.recentUseApps = apps;
    },


    SET_CURRENT_APP(state, app) {
        state.currentApp = app;
    },

    SET_VISIT_APP(state, app) {
        state.visitApp = app;
    },
    SET_VISIT_APP_PERMS(state, perms) {
        state.visitAppPermissions = perms;
    },
    SET_VISIT_APP_CURRENT_PERM(state, perm) {
        state.visitAppCurrentPerm = perm;
    },
}


const actions = {
    // 获取应用基本信息
    getData({state, commit}, moduleName) {
        return new Promise((resolve, reject) => {
            getBaseData(moduleName).then((data) => {
                commit("SET_DB", data)
                setData(moduleName, data)
                resolve()
            })
        }).catch(ex => {
            console.log(ex);
        })
    },

    // 根据桌面ID获取应用，此方法提供给移动端实时获取桌面下的应用。
    getDesktopApp({state, commit}, playload) {
        let {desktopCode, moduleName} = playload;
        return new Promise((resolve, reject) => {
            queryDesktopAppsByCode(desktopCode).then(res => {
                if (res.state === 200) {
                    let appDBItem = {
                        code: "APP",
                        version: "always_update",
                        value: res.data
                    };
                    if (state.db) {
                        state.db.push(appDBItem)
                    }
                    commit("SET_DB", state.db)
                    setData(moduleName, state.db)
                }
                resolve()
            });
        }).catch(ex => {
            console.log(ex);
        })
    },

    // 初始化应用信息
    initAppData({dispatch, state, commit, getters, rootGetters}, clientType = null) {
        let testMenuIcon = null;

        return new Promise((resolve, reject) => {
            if (clientType == null) {
                reject("未选择端，无法构造应用数据")
            }

            // 应用类型树形结构
            let appCategories = state.db.find(item => item.code == "APP_CATEGORY").value;
            let appCategoryTree = [];
            if (appCategories && appCategories.length > 0) {
                let mainCategories = appCategories.filter(item => item.parentId == "-1").map(item => {
                    return {
                        id: item.id,
                        name: item.name,
                        nodeType: "main-category",
                        hasChildren: false,
                        hover: false,
                        icon: item.icon,
                        children: [{
                            id: "all",
                            name: "全部",
                            hasChildren: false,
                            selected: false,
                            icon: null,
                            children: []
                        }]
                    }
                });
                mainCategories.forEach(mainCategory => {
                    // 获取子分类
                    let subCategories = appCategories.filter(item => item.parentId == mainCategory.id).map(item => {
                        return {
                            id: item.id,
                            name: item.name,
                            nodeType: "sub-category",
                            hasChildren: false,
                            selected: false,
                            icon: testMenuIcon,
                            children: []
                        }
                    })
                    mainCategory.children = mainCategory.children.concat(subCategories)
                    // 将大类加入到类树结构
                    appCategoryTree.push(mainCategory);
                })
            }
            commit("SET_APP_CATEGORY_TREE", appCategoryTree)

            // 生成应用标签树形结构
            let appTags = state.db.find(item => item.code == "APP_TAG").value;
            let appTagTree = appTags.filter(item => item.typeId == -1).map(item => {
                return {id: item.id, name: item.name, children: []}
            })
            appTagTree.forEach(tagType => {
                tagType.children = appTags.filter(item => item.typeId == tagType.id)
            })
            commit("SET_APP_TAG_TREE", appTagTree)


            // 获取应用数据源
            let apps = state.db.find(item => item.code == "APP").value;
            // 以下ID应该用store.userInfo中获取
            let assignAppIds = rootGetters['user/userApp'].assignedIdList;
            let savedAppIds = rootGetters['user/userApp'].favouriteIdList;
            let recommedAppIds = rootGetters['user/userApp'].recommendIdList;
            let recentUseAppIds = rootGetters['user/userApp'].recentIdList;

            // 授权应用
            let assignedApps = []
            apps.forEach(app => {
                app["nodeType"] = "app";
                app["isFavorite"] = false;
                app["isRecentUse"] = false;
                app["isRecommend"] = false;
                app["isAssigned"] = assignAppIds.indexOf(app.id) != -1 ? true : false;
                app["client"] = getAppClientInfo(app); // 根据App Type以及Client Type获取相关信息
            })

            // 根据客户端，再次过滤而授权的应用。
            if (clientType == "pc") {
                assignedApps = apps.filter(item => item.isAssigned && (item.clientType == null || item.clientType == "1") && item.pcAvailable == '1');
            } else if (clientType == "mobile") {
                assignedApps = apps.filter(item => item.isAssigned && (item.clientType == null || item.clientType == "2") && item.mobileAvailable == '1');
            }

            // 收藏应用，在已经授权的应用中过滤。
            let favoriteApps = [];
            savedAppIds.forEach(item => {
                assignedApps.forEach(app => {
                    if (app.id == item) {
                        app["isFavorite"] = true;
                        favoriteApps.push(app);
                    }
                })
            })

            // 推荐应用，在已经授权的应用中过滤。
            let recommendedApps = [];
            recommedAppIds.forEach(item => {
                assignedApps.forEach(app => {
                    if (app.id == item) {
                        app["isRecommend"] = true;
                        recommendedApps.push(app);
                    }
                })
            })

            // 最近使用应用，在已经授权的应用中过滤。
            let recentUseApps = [];
            recentUseAppIds.forEach(item => {
                assignedApps.forEach(app => {
                    if (app.id == item) {
                        app["isRecentUse"] = true;
                        recentUseApps.push(app);
                    }
                })
            })

            // 写入vuex
            commit("SET_ASSIGNED_APPS", assignedApps)
            commit("SET_FAVORITE_APPS", favoriteApps)
            commit("SET_RECOMMENDED_APPS", recommendedApps)
            commit("SET_RECENT_USE_APPS", recentUseApps)


            // 应用树形结构
            dispatch("genAppTree", {
                appCategoryTree,
                assignedApps,
                appType: null
            }).then(appTree => {
                commit("SET_APP_TREE", appTree)
            })

            // 流程应用树形结构
            dispatch("genAppTree", {
                appCategoryTree,
                assignedApps,
                appType: "1"
            }).then(appTree => {
                commit("SET_WORKFLOW_APP_TREE", appTree)
            })

            // 完成处理
            resolve()
        });
    },

    // 生成应用树形结构
    // appType=1是流程类应用
    // playload = {appCategoryTree:[], assignedApps:[], appType:null}
    // appCategoryTree = [], assignedApps=[], appType=null
    genAppTree(store, playload) {
        return new Promise((resolve, reject) => {
            // 获取数据
            let {appCategoryTree, assignedApps, appType} = playload;
            // 流程、数据模版类型的应用树形结构
            // 这只一种深度复制方法
            let appTree = JSON.parse(JSON.stringify(appCategoryTree));
            appTree.forEach(mainCategory => {
                // 获取应用
                mainCategory.children.forEach(subCategory => {
                    // 获取子类型的应用
                    if (appType) {
                        subCategory.children = assignedApps.filter(item => item.type == appType && item.categoryId == subCategory.id)
                    } else {
                        subCategory.children = assignedApps.filter(item => item.categoryId == subCategory.id)
                    }
                    // 查看子分类是否要显示，如果没有子元素
                    subCategory.hasChildren = subCategory.children.length > 0 ? true : false;
                })
                // 查看主分类是否要显示，如果没有子元素
                mainCategory.hasChildren = mainCategory.children.filter(subCategory => subCategory.hasChildren == true).length > 0 ? true : false
                // 第一个有应用的子类为默认选中的类
                let firstHasAppSubCategory = mainCategory.children.find(subCategory => subCategory.hasChildren == true);
                if (firstHasAppSubCategory) {
                    mainCategory.activeTabName = firstHasAppSubCategory.id;
                }
            })

            // 返回数据
            resolve(appTree);
        })
    },

    // 收藏应用
    saveApp({commit, dispatch, getters}, appId) {
        return new Promise((resolve, reject) => {
            saveApp(appId).then(resp => {
                if (resp.state == 200) {
                    resolve(resp);
                } else {
                    reject(resp);
                }
            }).catch(ex => {
                reject(ex);
            })
        })

    },

    // 取消收藏应用
    cancelSaveApp({commit, dispatch}, appId) {
        return new Promise((resolve, reject) => {
            cancelSaveApp(appId).then(resp => {
                if (resp.state == 200) {
                    resolve(resp);
                } else {
                    reject(resp);
                }
            })
        }).catch(ex => {
            reject(ex);
        })
    },

    // 获取当前访问的应用以及权限
    getVisitApp({commit, dispatch}, appKey) {
        return new Promise((resolve, reject) => {
            getAppByKey(appKey).then(resp => {
                if (resp.state == 200) {
                    // 获取应用
                    let visitApp = resp.data;
                    commit('SET_VISIT_APP', visitApp);

                    // 获取应用权限
                    let visitAppPerms = resp.data.perms.perms;
                    commit('SET_VISIT_APP_PERMS', visitAppPerms);
                    setPerms(visitAppPerms);

                    // 设置当前权限
                    // 判断当前缓存是否已存在权限，且是否相等
                    let arr = Object.keys(state.visitAppPermissions)
                    let hasExsited = false
                    if (arr.length === 0) {
                        commit('SET_VISIT_APP_CURRENT_PERM', {});
                        commit('SET_VISIT_APP_PERMS', []);
                        removePerms(STRING_CURRENT_PERMISSION_KEY)
                        removePerms(STRING_PERMISSION_KEY)
                    } else {
                        arr.forEach(permCode => {
                            if (state.visitAppCurrentPerm && state.visitAppCurrentPerm.id === state.visitAppPermissions[permCode].id) {
                                hasExsited = true
                            }
                        })
                    }
                    if (!hasExsited || !state.visitAppCurrentPerm) {
                        let perms = Object.values(visitAppPerms);
                        for (let key in perms) {
                            if (perms[key].hasPerm == true && perms[key].type == "1") {
                                commit('SET_VISIT_APP_CURRENT_PERM', perms[key]);
                                setCurrentPerm(STRING_CURRENT_PERMISSION_KEY, perms[key])
                                break;
                            }
                        }
                    }
                    // 处理完成
                    resolve(resp)
                } else {
                    commit('SET_VISIT_APP_CURRENT_PERM', {});
                    commit('SET_VISIT_APP_PERMS', []);
                    removePerms(STRING_CURRENT_PERMISSION_KEY)
                    removePerms(STRING_PERMISSION_KEY)
                    reject(resp)
                }
            }).catch(ex => {
                commit('SET_VISIT_APP_CURRENT_PERM', {});
                commit('SET_VISIT_APP_PERMS', []);
                removePerms(STRING_CURRENT_PERMISSION_KEY)
                removePerms(STRING_PERMISSION_KEY)
                console.log(ex)
                reject(ex)
            })
        })
    },

    setCurrentVisitAppPerm({commit}, data) {
        commit('SET_VISIT_APP_CURRENT_PERM', data);
        setCurrentPerm(STRING_CURRENT_PERMISSION_KEY, data)
    }


}
export default {
    namespaced: true,
    state,
    getters,
    mutations,
    actions
}
