import log from './util.log.js'
import cookies from './util.cookies.js'
// import XSS from 'xss'
const rtrim = /^[\s\uFEFF\xA0]+|[\s\uFEFF\xA0]+$/g

const util = {
  cookies,
  log,
  /**
   * 判断是否为空
   * @param {*} obj
   */
  isEmpty: function (obj, allowBlank = false) {
    if (util.isNull(obj)) return true
    if (util.isArray(obj)) return obj.length === 0
    if (util.isString(obj)) return (!(allowBlank || obj.length > 0))
    if (util.isObject(obj)) return util.isEmptyObject(obj)
    for (var key in obj) if (obj.hasOwnProperty(key)) return false
    return obj === undefined || (!allowBlank ? obj === '' : false)
  },
  /**
   * 判断是否不为空
   * @param {*} obj
   */
  isNotEmpty: function (obj, allowBlank = false) {
    return !util.isEmpty(obj, allowBlank)
  },
  /**
   * 判断是否为空对象
   * @param {*} obj
   */
  isEmptyObject: function (obj) {
    if (!obj) return true
    for (const name in obj) {
      return false
    }
    return true
  },
  /**
   * 判断是否为不空对象
   * @param {*} obj
   */
  isNotEmptyObject: function (obj) {
    return util.isNotEmptyObject(obj)
  },
  /**
   * 是否是对象
   * @param {*} input
   */
  isObject: function (input) {
    return Object.prototype.toString.call(input) === '[object Object]'
  },
  /**
   * 是否是数组
   * @param {*} input
   */
  isArray: function (input) {
    return input instanceof Array || Object.prototype.toString.call(input) === '[object Array]'
  },
  isDate: function (input) {
    return input instanceof Date || Object.prototype.toString.call(input) === '[object Date]'
  },
  isNumber: function (input) {
    return input instanceof Number || Object.prototype.toString.call(input) === '[object Number]'
  },
  isString: function (input) {
    return input instanceof String || Object.prototype.toString.call(input) === '[object String]'
  },
  isBoolean: function (input) {
    return typeof input === 'boolean'
  },
  isFunction: function (input) {
    return typeof input === 'function'
  },
  isNull: function (input) {
    return input === undefined || input === null
  },
  isNum: function (input) {
    if (util.isEmpty(input)) {
      return false
    } else {
      if (util.isString(input)) {
        input = Number(input)
      }
      if (isNaN(input)) {
        return false
      }
      return util.isNumber(input)
    }
  },
  isValidNumber: function (t) {
    return typeof t === 'number' && !isNaN(t) && isFinite(t)
  },
  isPlainObject: function (obj) {
    if (obj && Object.prototype.toString.call(obj) === '[object Object]' && obj.constructor === Object && !hasOwnProperty.call(obj, 'constructor')) {
      var key
      for (var k in obj) {
        key = k
      }
      return key === undefined || hasOwnProperty.call(obj, key)
    }
    return false
  },
  isJSON: function (str) {
    if (util.isString(str)) {
      try {
        const obj = JSON.parse(str)
        return util.isPlainObject(obj) || util.isArray(obj)
      } catch (e) {
        return false
      }
    }
    return false
  },
  trim: function (text) {
    return text == null ? '' : (text + '').replace(rtrim, '')
  },
  /**
   * 判断参数是否是其中之一
   */
  oneOf: function (value, validList) {
    for (let i = 0; i < validList.length; i++) {
      if (value === validList[i]) {
        return true
      }
    }
    return false
  },
  /**
   * 判断参数是否是数组对象其中之一
   */
  oneOfObj: function (value, validList, key) {
    for (let i = 0; i < validList.length; i++) {
      if (value === validList[i][key]) {
        return true
      }
    }
    return false
  },
  /**
   * 全局唯一标识符
   */
  guid: function () {
    return 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function (c) {
      const r = Math.random() * 16 | 0
      const v = c === 'x' ? r : (r & 0x3 | 0x8)
      return v.toString(16)
    })
  },
  /**
   * 转boolean 值
   * @param {*} obj
   * @param {*} defaultValue
   */
  toBoolean: function (obj, defaultValue = false) {
    if (util.isEmpty(obj)) {
      return defaultValue
    }
    return util.oneOf(obj, [true, 'True', 'Yes', 'true', '1', 1, 'yes', 'Y', 'y', 'T', 't'])
  },
  /**
 * 创建新数据，避免对象引用
 * @param {*} data
 * @param {*} defaultValue
 */
  newData: function (data, defaultValue) {
    return util.isNotEmpty(data) ? JSON.parse(JSON.stringify(data)) : (defaultValue || data)
  },
  /**
 * 转换json字符串的转换
 * @param {*} data
 * @param {*} defaultValue
 */
  parseData: function (data, defaultValue) {
    if (util.isNotEmpty(data)) {
      // eslint-disable-next-line no-eval
      return util.isPlainObject(data) || util.isArray(data) ? data : window.eval('(' + data + ')')
    } else {
      return (defaultValue || data)
    }
  },
  /**
 * 转换json字符串的转换
 * @param {*} data
 * @param {*} defaultValue
 */
  parseJSON: function (data, defaultValue) {
    if (util.isNotEmpty(data)) {
      return util.isJSON(data) ? JSON.parse(data) : data
    } else {
      return (defaultValue || data)
    }
  },
  /**
  * eval 数据
  */
  evalData: function (data) {
    // eslint-disable-next-line no-eval
    return window.eval(data)
  },
  /**
   * 格式化文件大小, 输出成带单位的字符串
   * @method formatSize
   * @grammar util.formatSize( size ) => String
   * @grammar util.formatSize( size, pointLength ) => String
   * @grammar util.formatSize( size, pointLength, units ) => String
   * @param {Number} size 文件大小
   * @param {Number} [pointLength=2] 精确到的小数点数。
   * @param {Array} [units=[ 'B', 'K', 'M', 'G', 'TB' ]] 单位数组。从字节，到千字节，一直往上指定。如果单位数组里面只指定了到了K(千字节)，同时文件大小大于M, 此方法的输出将还是显示成多少K.
   * @example
   * console.log( util.formatSize( 100 ) );    // => 100B
   * console.log( util.formatSize( 1024 ) );    // => 1.00K
   * console.log( util.formatSize( 1024, 0 ) );    // => 1K
   * console.log( util.formatSize( 1024 * 1024 ) );    // => 1.00M
   * console.log( util.formatSize( 1024 * 1024 * 1024 ) );    // => 1.00G
   * console.log( util.formatSize( 1024 * 1024 * 1024, 0, ['B', 'KB', 'MB'] ) );    // => 1024MB
   */
  formatSize: function (size, pointLength, units) {
    units = units || ['B', 'K', 'M', 'G', 'TB']
    let unit
    while ((unit = units.shift()) && size > 1024) {
      size = size / 1024
    }
    return (unit === 'B' ? size : size.toFixed(pointLength || 2)) + unit
  },
  /**
   * 格式化文本
   * @param {*} text
   */
  // formatText: function(text) {
  //   return text !== null ? ('' + XSS(text)).replace(/([^>\r\n]?)(\r\n|\n\r|\r|\n)/g, '$1' + '<br />' + '$2') : ''
  // },
  /**
   * @description 打开新页面
   * @param {String} url 地址
   */
  open: function (url) {
    var a = document.createElement('a')
    a.setAttribute('href', url)
    a.setAttribute('target', '_blank')
    a.setAttribute('id', 'ibps-open-link')
    document.body.appendChild(a)
    a.click()
    document.body.removeChild(document.getElementById('ibps-open-link'))
  },
  /**
 * 将array递归为一维数组。
 * @param {*} ary
 * @param {*} predicate
 * @param {*} result
 */
  flatten: function (ary, predicate, result) {
    result = result || []
    if (ary) {
      for (let i = 0; i < ary.length; i++) {
        const value = ary[i]
        if (Array.isArray(value)) {
          util.flatten(value, predicate, result)
        } else {
          (predicate && !predicate(value)) || result.push(value)
        }
      }
    }
    return result
  },

  /**
   * 想要执行业务代码，但又要等某个条件成熟，然后如果等了一段时间还不达到条件那就算了，就用这个工具方法
   * @param {Function} termFunc 条件检查方法，返回 true/false
   * @param {Function} businessFunc 条件方法返回true的时候执行该业务方法
   * @param {Number} interval 多久检查一次条件是否满足
   * @param {Number} timeout 等待多久超时后就不等了，停止
   */
  callAndWait(termFunc = () => true, businessFunc = () => { }, interval = 100, timeout = 5000) {
    let countTimeout = 0
    const callSelf = () => {
      setTimeout(() => {
        // 条件是否达到
        if (termFunc()) {
          // 执行业务代码
          businessFunc()
        } else {
          // 超时终止等待
          if (countTimeout >= timeout) {
            return
          }
          // 计时
          countTimeout += interval
          // 继续等待判断是否满足条件
          callSelf()
        }
      }, interval)
    }
    // 执行等待方法
    callSelf()
  },

  /**
   * 根据key获取对象的值
   * 用于解决key值大小写不同的问题
   * @param {Object} object 需要从中获取值的对象
   * @param {String} key 想要用什么key获取
   * @example
   * const value = this.$utils.getObjectValueByKey(data, 'id_')
   */
  getObjectValueByKey(object, key, defaultValue = '') {
    // 防获取对象则返回默认值
    if (util.isEmpty(object)) {
      return defaultValue
    }
    // 创建一个忽略大小写的正则对象
    const regx = new RegExp(`^${key}$`, 'gi')
    // 循环正则匹配
    for (const objectKey in object) {
      // 匹配成功返回值
      if (regx.test(objectKey)) {
        return object[objectKey]
      }
    }
    return defaultValue
  },

  /**
   * 在对象数组中查找指定字段值相匹配的元素
   * @param {Array} oriData 被查找列表
   * @param {Object} matchData 查找参考对象
   * @param {String} matchField 通过什么字段进行匹配，比如id_
   * @example
   * const item = this.$utils.getEqualItemInArrayByFieldName(listData, selectItem, 'id_')
   * if (item) {
   *   console.log('列表中存在selectItem元素')
   * }
   */
  getEqualItemInArrayByFieldName(oriData, matchData, matchField) {
    const matchPk = util.getObjectValueByKey(matchData, matchField)
    return oriData.find((data) => {
      const currentPk = util.getObjectValueByKey(data, matchField)
      return currentPk && matchPk && currentPk === matchPk
    })
  },

  /**
   * 数组去重方法
   */
  uniqueByKey(arr, key) {
    let hash = {};
    let result = arr.reduce((total, currentValue) => {
      if (!hash[currentValue[key]]) { //如果当前元素的key值没有在hash对象里，则可放入最终结果数组
        hash[currentValue[key]] = true; //把当前元素key值添加到hash对象
        total.push(currentValue); //把当前元素放入结果数组
      }
      return total; //返回结果数组
    }, []);
    return result;
  },


  /**
 * 判断路由拦截器刷新的时候页面是否一致
 * 如果两个appKey不一致，则需要刷新
 */
  isAppKeyEqual(to, from) {
    let afterAppKey = null;
    let beforAppKey = null;
    try {
      if (to && to.meta && to.meta.appKey) {
        afterAppKey = to.meta.appKey; //现在的key
      }

      if (from && from.meta && from.meta.appKey) {
        beforAppKey = from.meta.appKey; //现在的key
      }
    } catch {}  
    // 相同为true，不同为false
    return afterAppKey == beforAppKey; 
  },


}

export default util
